@@ -427,7 +427,6 @@ void MainWindow::initDocks()
 
     QList<IaitoDockWidget *> ioDocks
         = {filesDock = new FilesWidget(this),
-           filesystemDock = new FilesystemWidget(this),
            binariesDock = new BinariesWidget(this),
            mapsDock = new MapsWidget(this)};
     QList<IaitoDockWidget *> infoDocks = {
@@ -484,6 +483,15 @@ void MainWindow::initDocks()
     ui->menuWindows->addActions(makeActionList(windowDocks2));
     ui->menuAddInfoWidgets->addActions(makeActionList(infoDocks));
     ui->menuAddIoWidgets->addActions(makeActionList(ioDocks));
+    QAction *actionFilesystem = new QAction("Filesystem", this);
+    connect(actionFilesystem, &QAction::triggered, this, [this]() {
+        if (!filesystemDock) {
+            filesystemDock = new FilesystemWidget(this);
+            addExtraWidget(filesystemDock);
+        }
+        filesystemDock->show();
+    });
+    ui->menuAddIoWidgets->addAction(actionFilesystem);
     ui->menuAddDebugWidgets->addActions(makeActionList(debugDocks));
 
     auto uniqueDocks = windowDocks + windowDocks2 + infoDocks + debugDocks;
