@@ -1,6 +1,8 @@
 #ifndef CUSTOM_ASSERT_H
 #define CUSTOM_ASSERT_H 1
 
-#define assert(x) if ((x)) {}
+#define assert(x) \
+    if ((x)) { \
+    }
 
 #endif
