@@ -64,7 +64,11 @@ void Dashboard::updateContents()
         setPlainText(this->ui->relroEdit, "N/A");
     }
 
-    setPlainText(this->ui->baddrEdit, RAddressString(item2["baddr"].toVariant().toULongLong()));
+    ut64 baddr = item2["baddr"].toVariant().toULongLong();
+    if (baddr == 0 || baddr == UT64_MAX) {
+        baddr = Core()->getConfigi("bin.laddr");
+    }
+    setPlainText(this->ui->baddrEdit, RAddressString(baddr));
 
     // set booleans
     setBool(this->ui->vaEdit, item2, "va");
@@ -94,7 +98,7 @@ void Dashboard::updateContents()
     // Add hashes as a pair of Hash Name : Hash Value.
     for (const QString &key : hashes.keys()) {
         // Create a bold QString with the hash name uppercased
-        QString label = QString("<b>%1:</b>").arg(key.toUpper());
+        QString label = QStringLiteral("<b>%1:</b>").arg(key.toUpper());
 
         // Define a Read-Only line edit to display the hash value
         QLineEdit *hashLineEdit = new QLineEdit();
@@ -138,10 +142,6 @@ void Dashboard::updateContents()
     setPlainText(ui->percentageLineEdit, QString::number(analinfo["percent"].toInt()) + "%");
 
     QStringList libs = Core()->cmdList("il");
-    if (!libs.isEmpty()) {
-        libs.removeFirst();
-        libs.removeLast();
-    }
 
     // dunno: why not label->setText(lines.join("\n")?
     while (ui->verticalLayout_2->count() > 0) {
